
#ifndef __ICGFX_EFFECT_BCB_H
#define __ICGFX_EFFECT_BCB_H

#ifdef CGFXAB_EXPORTS
#define CGFXDLL_API __declspec(dllexport)
#else
#define CGFXDLL_API __declspec(dllimport)
#endif

#include <objbase.h>


#include "cgfx_stddefs.h"
#define CGFX_MAX_DIMENSIONS 4

#ifndef CGFXDLL_API
#define CGFXDLL_API
#endif

#ifdef WIN32
#define CGFXDLL_EXPORT __cdecl
#else
#define CGFXDLL_EXPORT
#endif // WIN32

//////////////////////////////////////////////////////////////////////////////
// Types /////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


typedef LPCSTR CGFXHANDLE;

enum CgFXPARAMETERTYPE {
    CgFXPT_UNKNOWN			= 0,
    CgFXPT_BOOL,
    CgFXPT_INT,
    CgFXPT_FLOAT,
    CgFXPT_DWORD,
    CgFXPT_STRING,

    CgFXPT_TEXTURE1D,
    CgFXPT_TEXTURE2D,
    CgFXPT_TEXTURERECT,
    CgFXPT_TEXTURECUBE,
    CgFXPT_TEXTURE3D,

    CgFXPT_SAMPLER1D,
    CgFXPT_SAMPLER2D,
    CgFXPT_SAMPLERRECT,
    CgFXPT_SAMPLERCUBE,
    CgFXPT_SAMPLER3D,

    CgFXPT_VERTEXSHADER,
    CgFXPT_PIXELSHADER,

    CgFXPT_STRUCT,
    
    // force 32-bit size enum
    CgFXPT_FORCE_DWORD      = 0x7fffffff
};

enum CgFXPARAMETERCLASS {
    CgFXPC_SCALAR,
    CgFXPC_VECTOR,
    CgFXPC_MATRIX_ROWS,
    CgFXPC_MATRIX_COLUMNS,
    CgFXPC_OBJECT,
    CgFXPC_STRUCT,

    CgFXPC_FORCE_DWORD      = 0x7fffffff
};

struct CgFXEFFECT_DESC {
    LPCSTR Creator;
    UINT Parameters;
    UINT Techniques;
    UINT Functions;
};

// CgFXPARAMETER_DESC::Flags
#define CGFX_PARAMETER_SHARED 1
#define CGFX_PARAMETER_LITERAL 2
#define CGFX_PARAMETER_ANNOTATION 4 

struct CgFXPARAMETER_DESC {
    LPCSTR Name;
    LPCSTR Semantic;
    CgFXPARAMETERCLASS Class;
    CgFXPARAMETERTYPE Type;
    UINT Dimension[CGFX_MAX_DIMENSIONS]; // 
    UINT Rows, Columns, Elements;
    UINT Annotations;
    UINT StructMembers;
    UINT Bytes;
    DWORD Flags;
};

struct CgFXTECHNIQUE_DESC {
    LPCSTR Name;
    UINT Annotations;
    UINT Passes;
};

struct CgFXVARYING {
    LPCSTR Name;
    UINT Resource;
    UINT Elements;
};

struct CgFXSAMPLER_INFO {
    UINT Type;
};

#define MAX_CGFX_DECL_LENGTH 16

struct CgFXPASS_DESC {
    LPCSTR Name;
    UINT Annotations;
    DWORD VSVersion;
    DWORD PSVersion;
    UINT VertexVaryingUsed;
    CgFXVARYING VertexVarying[MAX_CGFX_DECL_LENGTH];
    UINT FragmentVaryingUsed;
    CgFXVARYING FragmentVarying[MAX_CGFX_DECL_LENGTH];
    UINT PSSamplersUsed;
    CgFXSAMPLER_INFO PSSamplers[16];
};

enum CgFXMode {
    CgFX_Unknown,
    CgFX_OpenGL,
    CgFX_Direct3D8,
    CgFX_Direct3D9
};

//////////////////////////////////////////////////////////////////////////////
// Base Effect ///////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// base interface. extended by Effect and EffectCompiler.
//////////////////////////////////////////////////////////////////////////////
class ICgFXBaseEffect
{
public:
	//virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void __RPC_FAR *__RPC_FAR *ppvObject) = 0;
	virtual HRESULT STDMETHODCALLTYPE QueryInterface( 
            /* [in] */ REFIID riid,
            /* [iid_is][out] */ void __RPC_FAR *__RPC_FAR *ppvObject) = 0;
	virtual HRESULT STDMETHODCALLTYPE AddRef(void) = 0;
	virtual HRESULT STDMETHODCALLTYPE Release(void) = 0;

    virtual CGFXHANDLE STDMETHODCALLTYPE GetAnnotation(CGFXHANDLE object, UINT num) = 0;
    virtual CGFXHANDLE STDMETHODCALLTYPE GetAnnotationByName(CGFXHANDLE object, LPCSTR name) = 0;

    virtual CGFXHANDLE STDMETHODCALLTYPE GetParameter(CGFXHANDLE parent, UINT index) = 0;
    virtual CGFXHANDLE STDMETHODCALLTYPE GetParameterByName(CGFXHANDLE parent, LPCSTR name) = 0;
    virtual CGFXHANDLE STDMETHODCALLTYPE GetParameterBySemantic(CGFXHANDLE parent, LPCSTR name) = 0;
    virtual CGFXHANDLE STDMETHODCALLTYPE GetParameterElement(CGFXHANDLE parent, UINT element) = 0;

    virtual CGFXHANDLE STDMETHODCALLTYPE GetPass(CGFXHANDLE technique, UINT index) = 0;
    virtual CGFXHANDLE STDMETHODCALLTYPE GetPassByName(CGFXHANDLE technique, LPCSTR name) = 0;

    virtual CGFXHANDLE STDMETHODCALLTYPE GetTechnique(UINT index) = 0;
    virtual CGFXHANDLE STDMETHODCALLTYPE GetTechniqueByName(LPCSTR name) = 0;

    virtual HRESULT STDMETHODCALLTYPE GetDesc(CgFXEFFECT_DESC* pDesc) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetParameterDesc(CGFXHANDLE pParameter, CgFXPARAMETER_DESC* pDesc) = 0;

    virtual HRESULT STDMETHODCALLTYPE GetTechniqueDesc(CGFXHANDLE pTechnique, CgFXTECHNIQUE_DESC* pDesc) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetPassDesc(CGFXHANDLE pPass, CgFXPASS_DESC* pDesc) = 0;

    virtual HRESULT STDMETHODCALLTYPE SetValue(CGFXHANDLE pName, LPCVOID pData, UINT Bytes) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetValue(CGFXHANDLE pName, LPVOID pData, UINT Bytes) = 0;

    virtual HRESULT STDMETHODCALLTYPE SetFloat(CGFXHANDLE pName, FLOAT f) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetFloat(CGFXHANDLE pName, FLOAT* f) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetFloatArray(CGFXHANDLE pName, FLOAT* f, UINT count) = 0;
    virtual HRESULT STDMETHODCALLTYPE SetFloatArray(CGFXHANDLE pName, const FLOAT* f, UINT count) = 0;

    virtual HRESULT STDMETHODCALLTYPE GetInt(CGFXHANDLE pName, int *value) = 0;
    virtual HRESULT STDMETHODCALLTYPE SetInt(CGFXHANDLE pName, int value) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetIntArray(CGFXHANDLE pName, int *value, UINT count) = 0;
    virtual HRESULT STDMETHODCALLTYPE SetIntArray(CGFXHANDLE pName, const int *value, UINT count) = 0;

    virtual HRESULT STDMETHODCALLTYPE SetVector(CGFXHANDLE pName, const float *pVector, UINT vecSize) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetVector(CGFXHANDLE pName, float *pVector, UINT vecSize) = 0;
    virtual HRESULT STDMETHODCALLTYPE SetVectorArray(CGFXHANDLE pName, const float *pVector, UINT vecSize, 
                                   UINT count) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetVectorArray(CGFXHANDLE pName, float *pVector, UINT vecSize, UINT count) = 0;

    virtual HRESULT STDMETHODCALLTYPE SetMatrix(CGFXHANDLE pName, const float* pMatrix, UINT nRows, UINT nCols) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetMatrix(CGFXHANDLE pName, float* pMatrix, UINT nRows, UINT nCols) = 0;
    virtual HRESULT STDMETHODCALLTYPE SetMatrixArray(CGFXHANDLE pName, const float* pMatrix, UINT nRows, UINT nCols,
                                   UINT count) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetMatrixArray(CGFXHANDLE pName, float* pMatrix, UINT nRows, UINT nCols,
                                   UINT count) = 0;

    virtual HRESULT STDMETHODCALLTYPE SetMatrixTranspose(CGFXHANDLE pName, const float* pMatrix, UINT nRows, UINT nCols) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetMatrixTranspose(CGFXHANDLE pName, float* pMatrix, UINT nRows, UINT nCols) = 0;
    virtual HRESULT STDMETHODCALLTYPE SetMatrixTransposeArray(CGFXHANDLE pName, const float *pMatrix, UINT nRows, UINT nCols,
                                            UINT count) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetMatrixTransposeArray(CGFXHANDLE pName, float *pMatrix, UINT nRows, UINT nCols,
                                            UINT count) = 0;
	
    virtual HRESULT STDMETHODCALLTYPE SetDWORD(CGFXHANDLE pName, DWORD dw) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetDWORD(CGFXHANDLE pName, DWORD* dw) = 0;

    virtual HRESULT STDMETHODCALLTYPE SetBool(CGFXHANDLE pName, bool bvalue) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetBool(CGFXHANDLE pName, bool* bvalue) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetBoolArray(CGFXHANDLE pName, bool *bvalue, UINT count) = 0;
    virtual HRESULT STDMETHODCALLTYPE SetBoolArray(CGFXHANDLE pName, const bool *bvalue, UINT count) = 0;
	
    virtual HRESULT STDMETHODCALLTYPE SetString(CGFXHANDLE pName, LPCSTR pString) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetString(CGFXHANDLE pName, LPCSTR* ppString) = 0;

    virtual HRESULT STDMETHODCALLTYPE SetTexture(CGFXHANDLE pName, DWORD textureHandle) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetTexture(CGFXHANDLE pName, DWORD* textureHandle) = 0;

    virtual HRESULT STDMETHODCALLTYPE SetVertexShader(CGFXHANDLE pName, DWORD vsHandle) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetVertexShader(CGFXHANDLE pName, DWORD* vsHandle) = 0;

    virtual HRESULT STDMETHODCALLTYPE SetPixelShader(CGFXHANDLE pName, DWORD psHandle) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetPixelShader(CGFXHANDLE pName, DWORD* psHandle) = 0;
};

enum CgFXPassFlags {
    CGFX_DONOTSAVESTATE = 1,
    CGFX_DONOTSAVESHADERSTATE = 2,
};

class ICgFXEffect : public ICgFXBaseEffect
{
public:
    virtual HRESULT STDMETHODCALLTYPE SetTechnique(CGFXHANDLE pTechnique) = 0;
    virtual CGFXHANDLE STDMETHODCALLTYPE GetCurrentTechnique() = 0;

    virtual HRESULT STDMETHODCALLTYPE ValidateTechnique(CGFXHANDLE technique) = 0;
    virtual HRESULT STDMETHODCALLTYPE FindNextValidTechnique(CGFXHANDLE hTechnique, CGFXHANDLE *pTechnique) = 0;

    virtual HRESULT STDMETHODCALLTYPE Begin(UINT* pPasses, DWORD Flags) = 0;
    virtual HRESULT STDMETHODCALLTYPE Pass(UINT passNum) = 0;
    virtual HRESULT STDMETHODCALLTYPE End() = 0;

    virtual HRESULT STDMETHODCALLTYPE CloneEffect(ICgFXEffect** ppNewEffect) = 0; 

    virtual HRESULT STDMETHODCALLTYPE GetDevice(LPVOID* ppDevice) = 0;

    virtual HRESULT STDMETHODCALLTYPE OnLostDevice() = 0;
    virtual HRESULT STDMETHODCALLTYPE OnResetDevice() = 0;
};

class ICgFXEffectCompiler : public ICgFXBaseEffect
{
public:
    virtual HRESULT STDMETHODCALLTYPE CompileEffect(const char **compilerArgs, ICgFXEffect** ppEffect, 
                                  const char** ppCompilationErrors) = 0;
};

#ifndef CGFXAB_EXPORTS
#define CgFXCreateEffect CgFXCreateEffect_BCB
#define CgFXCreateEffectFromFileA CgFXCreateEffectFromFileA_BCB
#define CgFXCreateEffectCompiler CgFXCreateEffectCompiler_BCB
#define CgFXCreateEffectCompilerFromFileA CgFXCreateEffectCompilerFromFileA_BCB
#define CgFXSetDevice CgFXSetDevice_BCB
#define CgFXFreeDevice CgFXFreeDevice_BCB
#define CgFXGetErrors CgFXGetErrors_BCB
#endif


extern "C" {

CGFXDLL_API HRESULT CGFXDLL_EXPORT CgFXCreateEffect_BCB(
        LPCSTR               pSrcData,
        const char            **compilerArguments,
        ICgFXEffect**        ppEffect,
        const char**         ppCompilationErrors);

CGFXDLL_API HRESULT CGFXDLL_EXPORT CgFXCreateEffectFromFileA_BCB(
        LPCSTR               pSrcFile,
        const char            **compilerArguments,
        ICgFXEffect**        ppEffect,
        const char**         ppCompilationErrors);

CGFXDLL_API HRESULT CGFXDLL_EXPORT CgFXCreateEffectCompiler_BCB(
        LPCSTR                pSrcData,
        const char            **compilerArguments,
        ICgFXEffectCompiler** ppEffectCompiler,
        const char**          ppCompilationErrors);

CGFXDLL_API HRESULT CGFXDLL_EXPORT CgFXCreateEffectCompilerFromFileA_BCB(
        LPCSTR                pSrcFile,
        const char            **compilerArguments,
        ICgFXEffectCompiler** ppEffectCompiler,
        const char**          ppCompilationErrors);

CGFXDLL_API HRESULT CGFXDLL_EXPORT CgFXSetDevice_BCB(const char* pDeviceName, LPVOID pDevice);
CGFXDLL_API HRESULT CGFXDLL_EXPORT CgFXFreeDevice_BCB(const char* pDeviceName, LPVOID pDevice);

CGFXDLL_API HRESULT CGFXDLL_EXPORT CgFXGetErrors_BCB(const char** ppErrors);

} // extern "C"

#endif
